from abilian import testing

from .app import Application


class ConfigForTests(testing.TestConfig):
    CELERY_ALWAYS_EAGER = True  # run tasks locally, no async
    CELERY_EAGER_PROPAGATES_EXCEPTIONS = True


class BaseTestCase(testing.BaseTestCase):
    application_class = Application
    config_class = ConfigForTests

    def get_setup_config(self):
        """Called before creating application class.
        """
        config = testing.BaseTestCase.get_setup_config(self)
        if hasattr(self, 'no_login'):
            config.NO_LOGIN = self.no_login

        return config



from __future__ import print_function

from abilian.sbe.testing import BaseTestCase
from abilian.web import url_for


class TestViews(BaseTestCase):
    init_data = True
    no_login = True

    def test_all_registered_urls(self):

        SKIP = frozenset([
            'admin.audit_search_users',
            'search.search_main',
            'search.live',
            'notifications.debug_social',
            'social.groups_json',
            'social.groups_new',
            'social.users_json',
            'social.users_dt_json',
            'communities.community_default_image',
            'images.user_default',
        ])

        rules = sorted(self.app.url_map.iter_rules(), key=lambda x: x.endpoint)
        for rule in rules:

            if "GET" not in rule.methods:
                continue

            if rule.arguments:
                continue

            # FIXME. Skip several exceptions.
            if rule.endpoint in SKIP:
                continue

            # These endpoints expect a parameter ('q') that we don't provide
            if rule.endpoint.endswith("list_json2"):
                continue

            url = url_for(rule.endpoint)

            if '/ajax/' in url:
                continue

            print(rule.endpoint, url)

            try:
                response = self.client.get(url)
                assert response.status_code in (200, 302), \
                  "Bad link: {} (status={})".format(url, response.status_code)
            except:
                print("Problem with url: {}".format(url))
                raise



# coding=utf-8
"""
Models for the Forum.

Note: a few features are planned but not implemented yet, and are commented
out.
"""
from __future__ import absolute_import

from itertools import chain

from sqlalchemy import Column, ForeignKey, Integer, Unicode, UnicodeText
from sqlalchemy.event import listens_for
from sqlalchemy.ext.hybrid import hybrid_property
from sqlalchemy.orm import backref, relationship

from abilian.core.entities import SEARCHABLE, Entity
from abilian.sbe.apps.communities.models import Community, CommunityIdColumn, \
    community_content
from abilian.sbe.apps.documents.models import BaseContent, CmisObject
from abilian.services.indexing.adapter import SAAdapter


class ThreadClosedError(RuntimeError):

    def __init__(self, thread):
        super(ThreadClosedError, self).__init__(
            'The thread {!r} is closed. No modification allowed on its posts: '
            'creation, edition, deletion'.format(thread),)
        self.thread = thread


@community_content
class Thread(Entity):
    """A thread contains conversations among forum participants.

    The discussions in a thread may be sorted in chronological order or threaded
    by reply.

    (= Thread in SIOC, Message in ICOM 1.0).
    """
    __tablename__ = 'forum_thread'

    community_id = CommunityIdColumn()
    #: The community this thread belongs to
    community = relationship(
        Community,
        primaryjoin=(community_id == Community.id),
        backref=backref(
            'threads', cascade="all, delete-orphan"))

    #: The thread title (aka subject)
    _title = Column(
        'title', Unicode(255), nullable=False, default=u"", info=SEARCHABLE)

    # title is defined has an hybrid property to allow name <-> title sync (2 way)
    @hybrid_property
    def title(self):
        return self._title

    @title.setter
    def title(self, title):
        # set title before setting name, so that we don't enter an infinite loop
        # with _thread_sync_name_title
        self._title = title
        if self.name != title:
            self.name = title

    posts = relationship(
        'Post',
        primaryjoin='Thread.id == Post.thread_id',
        order_by='Post.created_at',
        cascade="all, delete-orphan",
        back_populates='thread',)

    @property
    def closed(self):
        """True if this thread doesn't accept more posts.
        """
        return self.meta.get('abilian.sbe.forum', {}).get('closed', False)

    @closed.setter
    def closed(self, value):
        self.meta.setdefault('abilian.sbe.forum', {})['closed'] = bool(value)
        self.meta.changed()

    def create_post(self, **kw):
        if self.closed:
            raise ThreadClosedError(self)

        kw['name'] = self.name
        post = Post(**kw)
        post.thread = self
        return post


@listens_for(Thread.name, "set", active_history=True)
def _thread_sync_name_title(entity, new_value, old_value, initiator):
    """Synchronize thread name -> title.

    thread.title -> name is done via hybrid_property, avoiding infinite
    loop (since "set" is received before attribute has received value)
    """
    if entity.title != new_value:
        entity.title = new_value
    return new_value


class Post(Entity):
    """A post is a message in a forum discussion thread.

    (= Post in DiscussionMessage in ICOM 1.0).
    """
    __tablename__ = 'forum_post'
    __indexable__ = False  # content is indexed at thread level

    #: The thread this post belongs to
    thread_id = Column(ForeignKey(Thread.id), nullable=False)
    thread = relationship(
        Thread, foreign_keys=thread_id, back_populates='posts')

    #: The post this post is a reply to, if any (currently not used)
    parent_post_id = Column(ForeignKey("forum_post.id"), nullable=True)
    parent_post = relationship("Post", foreign_keys=[parent_post_id])

    #: Markup type (Markdown, Textile...)
    # TODO: markup langage selection + default
    # markup_type = Column(String, default="Markdown")

    #: Source (markup) for the post
    # body_src = Column(UnicodeText, default=u"", nullable=False)

    #: HTML rendering of the post
    body_html = Column(UnicodeText, default=u"", nullable=False)

    @hybrid_property
    def title(self):
        return self.name

    @property
    def history(self):
        return self.meta.get('abilian.sbe.forum', {}).get('history', [])


class ThreadIndexAdapter(SAAdapter):
    """Index a thread and its posts.
    """

    @staticmethod
    def can_adapt(obj_cls):
        return obj_cls is Thread

    def get_document(self, obj):
        kw = super(ThreadIndexAdapter, self).get_document(obj)
        kw['text'] = u' '.join(
            chain((kw['text'],), [p.body_html for p in obj.posts]))
        return kw


# event listener to sync name with thread's name
@listens_for(Thread.name, "set", active_history=True)
def _thread_sync_name(thread, new_value, old_value, initiator):
    """Synchronize name with thread's name.
    """
    if new_value == old_value:
        return new_value

    for post in thread.posts:
        post.name = new_value
    return new_value


@listens_for(Post.thread, "set", active_history=True)
def _thread_change_sync_name(post, new_thread, old_thread, initiator):
    """Change name on thread change.
    """
    if new_thread == old_thread or new_thread is None:
        return new_thread
    post.name = new_thread.name
    return new_thread


@listens_for(Thread.posts, 'append')
@listens_for(Thread.posts, 'remove')
@listens_for(Thread.posts, 'set')
def _guard_closed_thread_collection(thread, value, *args):
    """Prevent add/remove/replace posts on a closed thread.
    """
    if isinstance(thread, Post):
        thread = thread.thread
        if thread is None:
            return

    if thread.closed:
        raise ThreadClosedError(thread)

    return value


class PostAttachment(BaseContent, CmisObject):
    __tablename__ = None  # type: str  # type: str
    __mapper_args__ = {'polymorphic_identity': 'forum_post_attachment'}
    sbe_type = 'forum_post:attachment'

    _post_id = Column(Integer, ForeignKey(Post.id), nullable=True)
    post = relationship(
        Post,
        primaryjoin=(_post_id == Post.id),
        backref=backref(
            'attachments',
            lazy='select',
            order_by='PostAttachment.name',
            cascade='all, delete-orphan'))



from flask_babel import lazy_gettext as _l

from abilian.core.util import BasePresenter


class CommunityPresenter(BasePresenter):

    @property
    def breadcrumbs(self):
        return [
            dict(
                label=_l("Communities"), path="/communities/"),
            dict(label=self._model.name)
        ]



# coding=utf-8

from __future__ import unicode_literals

from flask import current_app, g, url_for
from flask_login import current_user

from abilian.i18n import _l
from abilian.services.security import Admin
from abilian.web.action import Action, FAIcon, actions


class CalendarAction(Action):

    def url(self, context=None):
        return url_for("." + self.name, community_id=g.community.slug)


class EventAction(CalendarAction):

    def pre_condition(self, context):
        event = context.get('object')
        return not not event

    def url(self, context=None):
        event = context.get('object')
        return url_for(
            "." + self.name, community_id=g.community.slug, event_id=event.id)


def is_admin(context):
    svc = current_app.services['security']
    return svc.has_role(current_user, Admin, object=context.get('object'))


_actions = [
    CalendarAction(
        'calendar:global', 'new_event', _l('Create a new event'), icon='plus'),
    CalendarAction(
        'calendar:global', 'index', _l('Upcoming events'), icon='list'),
    EventAction(
        'calendar:event', 'event', _l('View event'), icon=FAIcon('eye')),
    EventAction(
        'calendar:event', 'event_edit', _l('Edit event'),
        icon=FAIcon('pencil')),
]


def register_actions(state):
    if not actions.installed(state.app):
        return
    with state.app.app_context():
        actions.register(*_actions)



from abilian.sbe.testing import BaseTestCase

from ..cmis.renderer import Feed, to_xml
from ..models import Document, Folder


class RendererTestCase(BaseTestCase):

    def test_folder_renderer(self):
        with self.app.test_client():
            folder = Folder(name="tototiti")
            result = to_xml(folder)
            assert "tototiti" in result
            assert "cmis:folder" in result

    def test_document_renderer(self):
        document = Document(name="tototiti")
        result = to_xml(document)
        assert "tototiti" in result
        assert "cmis:document" in result
        assert "<?xml" in result

        result = to_xml(document, no_xml_header=True)
        assert "<?xml" not in result

    def test_feed_renderer(self):
        folder = Folder(title="Toto Titi")
        document = Document(title="tatatutu")
        feed = Feed(folder, [document])
        result = feed.to_xml()

        assert "Toto Titi" in result
        assert "tatatutu" in result



# coding=utf-8

from sqlalchemy.exc import IntegrityError

from abilian.core.extensions import db
from abilian.sbe.testing import BaseTestCase

from ..models import Document, Folder


class TestService(BaseTestCase):

    def setUp(self):
        super(TestService, self).setUp()
        self.repository = self.app.extensions['content_repository']
        self.root = Folder(title=u"")
        self.session.add(self.root)
        self.session.flush()

    def test_create_doc(self):
        root = self.root
        doc = root.create_document(u"doc")
        db.session.flush()
        assert doc.path == "/doc"
        assert len(root.children) == 1
        assert not doc.is_root_folder

    def test_create_folder(self):
        root = self.root
        folder = root.create_subfolder(u"folder")
        db.session.flush()
        self.assertEqual(folder.title, u'folder')
        self.assertEqual(folder.name, u'folder')
        assert folder.path == "/folder"
        assert len(root.children) == 1
        assert len(folder.children) == 0
        assert not folder.is_root_folder

    def test_move(self):
        root = self.root
        doc = root.create_document(u"doc")
        folder = root.create_subfolder(u"folder")

        self.repository.move_object(doc, folder, u"newdoc")
        assert doc in folder.children
        assert doc not in root.children
        assert len(root.children) == 1
        assert len(folder.children) == 1
        assert doc.title == "newdoc"
        assert doc.path == "/folder/newdoc"

    def test_copy(self):
        root = self.root
        doc = root.create_document(u"doc")
        folder = root.create_subfolder(u"folder")

        doc_copy = self.repository.copy_object(doc, folder, u"copydoc")

        assert len(root.children) == 2
        assert len(folder.children) == 1
        self.assertEqual(doc_copy.title, "copydoc")
        self.assertEqual(doc_copy.name, "copydoc")
        assert doc_copy.path == "/folder/copydoc"
        assert doc_copy in folder.children

    def test_copy_nested_folders(self):
        root = self.root
        folder1 = root.create_subfolder(u"folder1")
        folder2 = root.create_subfolder(u"folder2")
        subfolder = folder1.create_subfolder(u"subfolder")

        folder1_copy = self.repository.copy_object(folder1, folder2)

        assert len(root.children) == 2
        assert folder1 in root.children
        assert folder2 in root.children

        assert len(folder1.children) == 1
        assert subfolder in folder1.children

        assert len(folder2.children) == 1
        assert folder1_copy in folder2.children

    def test_move_nested_folders(self):
        root = self.root
        folder1 = root.create_subfolder(u"folder1")
        folder2 = root.create_subfolder(u"folder2")
        subfolder = folder1.create_subfolder(u"subfolder")  # noqa

        self.repository.move_object(folder1, folder2)

        assert len(root.children) == 1
        assert len(folder1.children) == 1

    def test_rename(self):
        root = self.root
        folder = root.create_subfolder(u"folder")
        doc = folder.create_document(u"doc")

        self.repository.rename_object(doc, u"doc1")
        self.assertEqual(doc.title, 'doc1')
        self.assertEqual(doc.name, 'doc1')
        assert doc.path == '/folder/doc1'

        self.repository.rename_object(folder, u"folder1")
        self.assertEqual(folder.title, 'folder1')
        self.assertEqual(folder.name, 'folder1')
        assert folder.path == '/folder1'
        assert doc.path == '/folder1/doc1'

    def test_delete(self):
        root = self.root

        doc = root.create_document(u"doc")
        folder = root.create_subfolder(u"folder")
        self.session.flush()

        assert doc.parent == root
        assert folder.parent == root
        assert doc in root.children
        assert folder in root.children
        assert not doc.is_root_folder
        assert not folder.is_root_folder

        doc_id = doc.id
        folder_id = folder.id

        self.repository.delete_object(doc)
        self.repository.delete_object(folder)
        assert len(root.children) == 0
        self.session.flush()

        assert Folder.query.get(folder_id) is None
        assert Document.query.get(doc_id) is None

        # test delete tree
        folder = root.create_subfolder(u"folder")
        sub = folder.create_subfolder(u'subfolder')  # noqa
        doc = folder.create_document(u"doc")  # noqa
        self.session.flush()

        self.repository.delete_object(folder)
        assert len(root.children) == 0
        assert Folder.query.all() == [root]
        assert Document.query.all() == []

    def test_no_duplicate_name(self):
        root = self.root
        root.create_subfolder(u'folder_1')
        root.create_subfolder(u'folder_1')
        try:
            self.session.flush()
            self.fail('Could create folders with duplicate name')
        except IntegrityError:
            pass



# coding=utf-8
"""
Forum views
"""
from __future__ import absolute_import, print_function, unicode_literals

from datetime import date, datetime

from flask import g, render_template
from toolz import groupby

from abilian.i18n import _l
from abilian.web import url_for, views
from abilian.web.action import ButtonAction

from ..communities.blueprint import Blueprint
from ..communities.views import default_view_kw
from .forms import EventForm
from .models import Event

blueprint = Blueprint(
    "calendar", __name__, url_prefix="/calendar", template_folder="templates")
route = blueprint.route


@route('/')
def index():
    events = Event.query \
        .filter(Event.end > datetime.now()) \
        .order_by(Event.start) \
        .all()

    def get_month(event):
        year = event.start.year
        month = event.start.month
        return date(year, month, 1)

    groups = sorted(groupby(get_month, events).items())
    ctx = {'groups': groups,}
    return render_template('calendar/index.html', **ctx)


@route('/archives/')
def archives():
    events = Event.query \
        .order_by(Event.start) \
        .filter(Event.end <= datetime.now()) \
        .all()

    def get_month(event):
        year = event.start.year
        month = event.start.month
        return date(year, month, 1)

    groups = sorted(groupby(get_month, events).items())
    ctx = {'groups': groups,}
    return render_template('calendar/archives.html', **ctx)


class BaseEventView(object):
    Model = Event
    Form = EventForm
    pk = 'event_id'
    base_template = 'community/_base.html'

    def index_url(self):
        return url_for(".index", community_id=g.community.slug)

    def view_url(self):
        return url_for(self.obj)


class EventView(BaseEventView, views.ObjectView):
    methods = ['GET', 'HEAD']
    Form = EventForm
    template = 'calendar/event.html'

    @property
    def template_kwargs(self):
        kw = super(EventView, self).template_kwargs
        kw['event'] = self.obj
        return kw


event_view = EventView.as_view('event')
views.default_view(
    blueprint, Event, 'event_id', kw_func=default_view_kw)(event_view)
route('/<int:event_id>/')(event_view)


class EventCreateView(BaseEventView, views.ObjectCreate):
    POST_BUTTON = ButtonAction(
        'form', 'create', btn_class='primary', title=_l('Post this event'))

    title = _l("New event")

    def after_populate_obj(self):
        if self.obj.community is None:
            self.obj.community = g.community._model

    def get_form_buttons(self, *args, **kwargs):
        return [self.POST_BUTTON, views.object.CANCEL_BUTTON]

    @property
    def activity_target(self):
        return self.obj.community


event_create_view = EventCreateView.as_view('new_event', view_endpoint='.event')
route('/new_event/')(event_create_view)


class EventEditView(BaseEventView, views.ObjectEdit):
    POST_BUTTON = ButtonAction(
        'form', 'create', btn_class='primary', title=_l('Post this event'))

    title = _l("Edit event")


event_edit_view = EventEditView.as_view('event_edit', view_endpoint='.event')
route('/<int:event_id>/edit')(event_edit_view)



#!./bin/python

import subprocess
import time

from six.moves.urllib.parse import urlopen

# Some random number
BIND = '0.0.0.0'
PORT = 4034
HOME = "http://{}:{}/".format(BIND, PORT)

subprocess.call(['./bin/pip', 'install', 'gunicorn'])

p = subprocess.Popen(
    ["./bin/gunicorn", 'wsgi:app', '-b', '{}:{}'.format(BIND, PORT)])

try:
    # Just in case
    time.sleep(5)
    page = urlopen(HOME).read()
    assert "Welcome to Abilian" in page
finally:
    p.terminate()
    p.wait()



# coding=utf-8
"""
"""
from __future__ import absolute_import, unicode_literals

import bleach
from wtforms import BooleanField, StringField, TextAreaField

from abilian.i18n import _l
from abilian.web.forms import Form, RichTextWidget
from abilian.web.forms.fields import FileField
from abilian.web.forms.filters import strip
from abilian.web.forms.validators import optional, required

ALLOWED_TAGS = [
    'a',
    'abbr',
    'acronym',
    'b',
    'blockquote',
    'br',
    'code',
    'em',
    'i',
    'li',
    'ol',
    'strong',
    'ul',
    'h1',
    'h2',
    'h3',
    'h4',
    'h5',
    'h6',
    'p',
    'u',
    'img',
]

ALLOWED_ATTRIBUTES = {
    '*': ['title'],
    'p': ['style'],
    'a': ['href', 'title'],
    'abbr': ['title'],
    'acronym': ['title'],
    'img': ['src', 'alt', 'title'],
}

ALLOWED_STYLES = ['text-align',]

WIDGET_ALLOWED = {}
for attr in ALLOWED_TAGS:
    allowed = ALLOWED_ATTRIBUTES.get(attr, True)
    if not isinstance(allowed, bool):
        allowed = {tag: True for tag in allowed}
    WIDGET_ALLOWED[attr] = allowed

# instantiate this one before PostForm fields, so that it is listed first
# when Threadform is displayed
_TITLE_FIELD = StringField(
    label=_l("Title"), filters=(strip,), validators=[required()])


class BasePostForm(Form):
    message = TextAreaField(
        label=_l("Message"),
        widget=RichTextWidget(allowed_tags=WIDGET_ALLOWED),
        filters=(strip,),
        validators=[required()])
    attachments = FileField(
        label=_l('Attachments'), multiple=True, validators=[optional()])

    def validate_message(self, field):
        field.data = bleach.clean(
            field.data,
            tags=ALLOWED_TAGS,
            attributes=ALLOWED_ATTRIBUTES,
            styles=ALLOWED_STYLES,
            strip=True)


class PostForm(BasePostForm):
    send_by_email = BooleanField(label=_l("Send by email?"), default=True)


class ThreadForm(PostForm):
    title = _TITLE_FIELD


class PostEditForm(BasePostForm):
    reason = StringField(
        label=_l('Reason'),
        description=_l('Description of your edit'),
        filters=(strip,),
        validators=(optional(),))



# coding=utf-8
from flask import url_for
from flask_babel import lazy_gettext as _l

from abilian.sbe.apps.communities.actions import CommunityEndpoint
from abilian.web.action import Action, FAIcon, ModalActionMixin, actions


class WikiPageAction(Action):
    Endpoint = CommunityEndpoint

    def pre_condition(self, context):
        page = context.get('object')
        return bool(page)

    def url(self, context=None):
        if self._url:
            return self._url
        else:
            page = context.get('object')
            kw = self.endpoint.get_kwargs()
            kw['title'] = page.title
            return url_for(self.endpoint.name, **kw)


class WikiPageModalAction(ModalActionMixin, WikiPageAction):
    pass


class WikiAction(Action):
    Endpoint = CommunityEndpoint


_actions = (
    WikiPageAction(
        'wiki:page', 'view', _l(u'View'), endpoint='.page', icon='eye-open'),
    WikiPageAction(
        'wiki:page', 'edit', _l(u'Edit'), endpoint='.page_edit', icon='pencil'),
    WikiPageModalAction(
        'wiki:page',
        'upload_attachment',
        _l(u'Upload an attachment'),
        url='#upload-files',
        icon='plus'),
    WikiPageAction(
        'wiki:page',
        'source',
        _l(u'Source'),
        endpoint='.page_source',
        icon=FAIcon('code')),
    WikiPageAction(
        'wiki:page',
        'changes',
        _l(u'Changes'),
        endpoint='.page_changes',
        icon='time'),
    WikiPageModalAction(
        'wiki:page', 'delete', _l(u'Delete'), url='#modal-delete',
        icon='trash'),
    WikiAction(
        'wiki:global',
        'new',
        _l(u'New page'),
        endpoint='.page_new',
        icon='plus'),
    WikiAction(
        'wiki:global',
        'pages',
        _l(u'All pages'),
        endpoint='.wiki_pages',
        icon='list'),
    WikiAction(
        'wiki:global',
        'help',
        _l(u'Syntax help'),
        endpoint='.wiki_help',
        icon='info-sign'),)


def register_actions(state):
    if not actions.installed(state.app):
        return

    with state.app.app_context():
        actions.register(*_actions)



# coding=utf-8
"""
"""
from __future__ import absolute_import

from flask import Blueprint as BaseBlueprint
from flask import g
from werkzeug.exceptions import NotFound

from abilian.i18n import _l
from abilian.web.action import Endpoint
from abilian.web.nav import BreadcrumbItem

from . import security
from .models import Community
from .presenters import CommunityPresenter


class Blueprint(BaseBlueprint):
    """
    Blueprint for community based views.

    It sets g.community and perform access verification for the traversed
    community.
    """
    _BASE_URL_PREFIX = '/communities'
    _ROUTE_PARAM = '<string:community_id>'

    def __init__(self, *args, **kwargs):
        url_prefix = kwargs.get('url_prefix', '')

        if kwargs.pop('set_community_id_prefix', True):
            if (not url_prefix) or url_prefix[0] != '/':
                url_prefix = '/' + url_prefix
            url_prefix = self._ROUTE_PARAM + url_prefix

        if not url_prefix.startswith(self._BASE_URL_PREFIX):
            if (not url_prefix) or url_prefix[0] != '/':
                url_prefix = '/' + url_prefix
            url_prefix = self._BASE_URL_PREFIX + url_prefix

        if url_prefix[-1] == '/':
            url_prefix = url_prefix[:-1]
        kwargs['url_prefix'] = url_prefix

        BaseBlueprint.__init__(self, *args, **kwargs)
        self.url_value_preprocessor(pull_community)
        self.url_value_preprocessor(init_current_tab)
        self.before_request(check_access)


def check_access():
    if hasattr(g, 'community'):
        # communities.index is not inside a community, for example
        security.check_access(g.community)


def init_current_tab(endpoint, values):
    """Ensure g.current_tab exists.
    """
    g.current_tab = None


def pull_community(endpoint, values):
    """url_value_preprocessor function.
    """
    g.nav['active'] = 'section:communities'
    g.breadcrumb.append(
        BreadcrumbItem(
            label=_l(u'Communities'), url=Endpoint('communities.index')))

    try:
        slug = values.pop('community_id')
        community = Community.query.filter(Community.slug == slug).first()
        if community:
            g.community = CommunityPresenter(community)
            wall_url = Endpoint('wall.index', community_id=community.slug)
            breadcrumb_item = BreadcrumbItem(label=community.name, url=wall_url)
            g.breadcrumb.append(breadcrumb_item)
        else:
            raise NotFound()
    except KeyError:
        pass



# coding=utf-8
"""

"""
from __future__ import absolute_import

import fileinput
import logging
import sys
from email.parser import FeedParser

from flask_script import Manager

from .tasks import check_maildir, process_email

logger = logging.getLogger(__name__)

manager = Manager(description='SBE forum commands', help='SBE forum commands')


@manager.option(
    '-f',
    '--filename',
    help='email filename; defaults to standard input',
    default=u'-',
    required=False,)
def inject_email(filename=u'-'):
    """Read one email from stdin, parse it, forward it in a celery task to be persisted."""

    parser = FeedParser()

    if logger.level is logging.NOTSET:
        logger.setLevel(logging.INFO)

    try:
        # iterate over stdin
        for line in fileinput.input(filename):
            parser.feed(line)
    except KeyboardInterrupt:
        logger.info('Aborted by user, exiting.')
        sys.exit(1)
    except:
        logger.error('Error during email parsing', exc_info=True)
        sys.exit(1)
    finally:
        # close the parser to generate a email.message
        message = parser.close()
        fileinput.close()

    if message:
        # make sure no email.errors are present
        if not message.defects:
            process_email.delay(message)
        else:
            logger.error(
                'email has defects, message content:\n'
                '------ START -------\n'
                '%s'
                '\n------ END -------\n',
                message,
                extra={'stack': True,})
    else:
        logger.error('no email was parsed from stdin', extra={'stack': True,})


@manager.command
def check_email():
    """
    Read one email from current user Maildir, parse it, forward it in a celery task to be persisted.
    """

    # FIXME: doesn't appear legit.
    check_maildir.delay()
